==> kdeutils-4.1.1.tar.bz2: kdeutils-4.1.1/okteta/program/oktetakakao/controllers/view/stringsextract/extractstringsjob.cpp <==
/*
    This file is part of the Okteta Kakao module, part of the KDE project.

    Copyright 2008 Friedrich W. H. Kossebau <kossebau@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) version 3, or any
    later version accepted by the membership of KDE e.V. (or its
    successor approved by the membership of KDE e.V.), which shall
    act as a proxy defined in Section 6 of version 3 of the license.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library. If not, see <http://www.gnu.org/licenses/>.
*/

#include "extractstringsjob.h"

// Okteta core
#include <kabstractbytearraymodel.h>
#include <kcharcodec.h>
#include <khechar.h>
// Qt
#include <QtGui/QApplication>


static const int ExtractStringBlockSize = 100000;

void ExtractStringsJob::exec()
{

    // reset
    mContainedStringList->clear();

    // check
    if( !mByteArrayModel || !mSelection.isValid() )
    {
        deleteLater(); // TODO: could be reused on next operation

        return;
    }

    bool stringStarted = false;
    int stringStart = mSelection.start();
    QString string;

    int i = mSelection.start();
    int blockEnd = mSelection.start();
    while( i<=mSelection.end() )
    {
        blockEnd += ExtractStringBlockSize;
        if( blockEnd > mSelection.end() )
            blockEnd = mSelection.end();

        for( ; i<=blockEnd; ++i )
        {
            const KHECore::KChar decodedChar = mCharCodec->decode( mByteArrayModel->datum(i) );
            // TODO: Zeilenumbrüche ausnehmen
            const bool isStringChar = ( !decodedChar.isUndefined() &&
                                        (decodedChar.isLetterOrNumber() || decodedChar.isSpace() || decodedChar.isPunct()) );

            if( isStringChar )
            {
                if( !stringStarted )
                {
                    stringStart = i;
                    stringStarted = true;
                    string.clear();
                }
                string.append( decodedChar );
            }
            else
            {
                if( stringStarted )
                {
                    if( i-stringStart >= mMinLength )
                        mContainedStringList->append( ContainedString(string,stringStart) );
                    stringStarted = false;
                }
            }
        }

        QApplication::processEvents( QEventLoop::ExcludeUserInputEvents | QEventLoop::ExcludeSocketNotifiers, 100 );
    }

    // last string not ended?
    if( stringStarted )
    {
        if( i-stringStart >= mMinLength )
            mContainedStringList->append( ContainedString(string,stringStart) );
        stringStarted = false;
    }

    deleteLater(); // TODO: could be reused on next operation
}

==> kdeutils-4.2.3.tar.bz2: kdeutils-4.2.3/okteta/program/oktetakakao/controllers/view/stringsextract/extractstringsjob.cpp <==
/*
    This file is part of the Okteta Kakao module, part of the KDE project.

    Copyright 2008 Friedrich W. H. Kossebau <kossebau@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) version 3, or any
    later version accepted by the membership of KDE e.V. (or its
    successor approved by the membership of KDE e.V.), which shall
    act as a proxy defined in Section 6 of version 3 of the license.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library. If not, see <http://www.gnu.org/licenses/>.
*/

#include "extractstringsjob.h"

// Okteta core
#include <abstractbytearraymodel.h>
#include <charcodec.h>
#include <khechar.h>
// Qt
#include <QtGui/QApplication>


static const int ExtractStringBlockSize = 100000;

void ExtractStringsJob::exec()
{

    // reset
    mContainedStringList->clear();

    // check
    if( !mByteArrayModel || !mSelection.isValid() )
    {
        deleteLater(); // TODO: could be reused on next operation

        return;
    }

    bool stringStarted = false;
    int stringStart = mSelection.start();
    QString string;

    int i = mSelection.start();
    int blockEnd = mSelection.start();
    while( i<=mSelection.end() )
    {
        blockEnd += ExtractStringBlockSize;
        if( blockEnd > mSelection.end() )
            blockEnd = mSelection.end();

        for( ; i<=blockEnd; ++i )
        {
            const KHECore::KChar decodedChar = mCharCodec->decode( mByteArrayModel->datum(i) );
            // TODO: Zeilenumbrüche ausnehmen
            const bool isStringChar = ( !decodedChar.isUndefined() &&
                                        (decodedChar.isLetterOrNumber() || decodedChar.isSpace() || decodedChar.isPunct()) );

            if( isStringChar )
            {
                if( !stringStarted )
                {
                    stringStart = i;
                    stringStarted = true;
                    string.clear();
                }
                string.append( decodedChar );
            }
            else
            {
                if( stringStarted )
                {
                    if( i-stringStart >= mMinLength )
                        mContainedStringList->append( ContainedString(string,stringStart) );
                    stringStarted = false;
                }
            }
        }

        QApplication::processEvents( QEventLoop::ExcludeUserInputEvents | QEventLoop::ExcludeSocketNotifiers, 100 );
    }

    // last string not ended?
    if( stringStarted )
    {
        if( i-stringStart >= mMinLength )
            mContainedStringList->append( ContainedString(string,stringStart) );
        stringStarted = false;
    }

    deleteLater(); // TODO: could be reused on next operation
}

#include "extractstringsjob.moc"
